package com.ubsidifinance.dailog

import android.os.Build
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.aspectRatio
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.kizitonwose.calendar.compose.HorizontalCalendar
import com.kizitonwose.calendar.compose.rememberCalendarState
import com.kizitonwose.calendar.core.CalendarDay
import com.kizitonwose.calendar.core.CalendarMonth
import com.kizitonwose.calendar.core.DayPosition
import com.kizitonwose.calendar.core.yearMonth
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.ui.theme.blue316AFF
import com.ubsidifinance.ui.theme.grey_B3B3B3
import com.ubsidifinance.ui.theme.transparent
import com.ubsidifinance.ui.theme.white
import com.ubsidifinance.utils.toFormat
import kotlinx.coroutines.launch
import java.time.DayOfWeek
import java.time.LocalDate
import java.time.YearMonth
import java.time.format.TextStyle
import java.util.Locale

@RequiresApi(Build.VERSION_CODES.O)
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CustomDatePicker(
    modifier: Modifier = Modifier,
    title: String = "Date From",
    onDismissRequest: () -> Unit,
    onConfirmation: (LocalDate) -> Unit,
) {

    val interaction = remember { MutableInteractionSource() }
    var selectedDate by remember { mutableStateOf<LocalDate?>(null) }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {
        Column(modifier = modifier.padding(32.dp)) {

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .fillMaxWidth()
                    .padding(bottom = 24.dp),
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                TextComponent(
                    text = title,
                    fontSize = 22.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surface
                )

                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = "Close Button",
                    modifier = modifier
                        .clip(CircleShape)
                        .clickable(
                            indication = null,
                            interactionSource = interaction,
                            onClick = onDismissRequest
                        )
                        .size(20.dp)
                        .padding(4.dp),
                    tint = MaterialTheme.colorScheme.inversePrimary
                )
            }

            CustomCalendar(onDateSelected = {
                selectedDate = it
            })

            ButtonComponent(
                modifier = modifier.padding(top = 24.dp),
                text = "Apply",
                onClick = {
                    onDismissRequest.invoke()
                    selectedDate?.let { onConfirmation.invoke(it) }
                }
            )
        }
    }
}

@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun CustomCalendar(
    modifier: Modifier = Modifier,
    onDateSelected: (LocalDate) -> Unit
) {
    val currentMonth = remember { YearMonth.now() }
    val startMonth = remember { currentMonth.minusMonths(100) }
    val endMonth = remember { currentMonth.plusMonths(100) }
    val firstDayOfWeek = remember { DayOfWeek.MONDAY }
    var selectedDate by remember { mutableStateOf<LocalDate?>(null) }
    val scope = rememberCoroutineScope()
    var isShowMonthPicker by remember { mutableStateOf(false) }
    var isShowYearPicker by remember { mutableStateOf(false) }

    val state = rememberCalendarState(
        startMonth = startMonth,
        endMonth = endMonth,
        firstVisibleMonth = currentMonth,
        firstDayOfWeek = firstDayOfWeek
    )

    if (isShowYearPicker) {
        YearPickerDialog(
            startYear = startMonth,
            endYear = endMonth,
            currentYear = selectedDate?.yearMonth ?: currentMonth,
            onDismissRequest = {
                isShowYearPicker = false
            }, onConfirmation = { year ->

                scope.launch {
                    state.scrollToMonth(
                        YearMonth.of(
                            year, selectedDate?.yearMonth?.month ?: currentMonth.month
                        )
                    )
                }
            })
    }

    if (isShowMonthPicker) {
        MonthPickerDialog(
            currentMonth = selectedDate?.yearMonth ?: currentMonth,
            onDismissRequest = {
                isShowMonthPicker = false
            }, onConfirmation = { month ->

                scope.launch {
                    state.scrollToMonth(
                        YearMonth.of(
                            selectedDate?.yearMonth?.year ?: currentMonth.year, month
                        )
                    )
                }
            })
    }

    HorizontalCalendar(

        state = state,
        dayContent = {
            Day(it, isSelected = selectedDate == it.date) { day ->
                selectedDate = day.date
                onDateSelected.invoke(day.date)
            }
        },
        monthHeader = { month ->

            DaysOfWeekTitle(
                month,
                onNextMonth = {
                    scope.launch {
                        state.scrollToMonth(month.yearMonth.plusMonths(1))
                    }
                },
                onPreviousMonth = {
                    scope.launch {
                        state.scrollToMonth(month.yearMonth.minusMonths(1))
                    }
                },
                onYearClick = {
                    isShowYearPicker = true
                },
                onMonthClick = {
                    isShowMonthPicker = true
                }
            )
        }
    )

}

@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun DaysOfWeekTitle(
    month: CalendarMonth,
    onNextMonth: () -> Unit,
    onPreviousMonth: () -> Unit,
    onMonthClick: () -> Unit,
    onYearClick: () -> Unit,
) {
    val daysOfWeek = remember { month.weekDays.first().map { it.date.dayOfWeek } }
    val interactionSource = remember { MutableInteractionSource() }

    Column {
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .padding(bottom = 18.dp),
            horizontalArrangement = Arrangement.SpaceEvenly
        ) {

            Icon(
                painter = painterResource(R.drawable.ic_back_arrow),
                contentDescription = null,
                modifier = Modifier
                    .clip(CircleShape)
                    .clickable(onClick = onPreviousMonth)
                    .size(24.dp)
                    .padding(5.dp),
                tint = MaterialTheme.colorScheme.inversePrimary
            )

            Row(verticalAlignment = Alignment.CenterVertically) {
                TextComponent(
                    text = "${
                        month.yearMonth.month.getDisplayName(
                            TextStyle.FULL,
                            Locale.getDefault()
                        )
                    } ",
                    fontSize = 22.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surface,
                    modifier = Modifier.clickable(
                        onClick = onMonthClick,
                        interactionSource = interactionSource,
                        indication = null
                    )
                )

                TextComponent(
                    text = "- ${month.yearMonth.year}",
                    fontSize = 22.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surface,
                    modifier = Modifier.clickable(
                        onClick = onYearClick,
                        interactionSource = interactionSource,
                        indication = null
                    )
                )
            }


            Icon(
                painter = painterResource(R.drawable.ic_back_arrow),
                contentDescription = null,
                modifier = Modifier
                    .rotate(180f)
                    .clip(CircleShape)
                    .clickable(onClick = onNextMonth)
                    .size(24.dp)
                    .padding(5.dp),
                tint = MaterialTheme.colorScheme.inversePrimary
            )

        }


        Row(modifier = Modifier.fillMaxWidth()) {
            for (dayOfWeek in daysOfWeek) {

                TextComponent(
                    modifier = Modifier.weight(1f),
                    text = dayOfWeek.getDisplayName(TextStyle.SHORT, Locale.getDefault()),

                    style = androidx.compose.ui.text.TextStyle(
                        fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                        color = MaterialTheme.colorScheme.surface,
                        fontSize = 16.sp,
                        textAlign = TextAlign.Center,

                        ),
                )
            }
        }
    }
}

@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun Day(day: CalendarDay, isSelected: Boolean, onClick: (CalendarDay) -> Unit) {
    Box(
        modifier = Modifier
            .aspectRatio(1f)
            .clip(RoundedCornerShape(8.dp))
            .padding(10.dp)
            .background(
                color = if (isSelected) blue316AFF else transparent,
                RoundedCornerShape(8.dp)
            )
            .clickable(
                enabled = day.position == DayPosition.MonthDate,
                onClick = { onClick(day) }
            ),
        contentAlignment = Alignment.Center
    ) {
        TextComponent(
            text = day.date.dayOfMonth.toString(),
            fontSize = 14.sp,
            fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400)),
            color = if (day.position == DayPosition.MonthDate) if (isSelected) white else MaterialTheme.colorScheme.surface else grey_B3B3B3
        )
    }
}