package com.ubsidifinance.dailog

import android.os.Build
import androidx.annotation.RequiresApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.itemsIndexed
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.ui.theme.blue316AFF
import com.ubsidifinance.ui.theme.transparent
import com.ubsidifinance.ui.theme.white
import java.time.YearMonth
import java.util.Locale

@RequiresApi(Build.VERSION_CODES.O)
@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun MonthPickerDialog(
    modifier: Modifier = Modifier,
    currentMonth: YearMonth = YearMonth.now(),
    onDismissRequest: () -> Unit,
    onConfirmation: (Int) -> Unit,
) {
    val interaction = remember { MutableInteractionSource() }
    var selectIndex by remember { mutableIntStateOf(currentMonth.month.value - 1) }
    val listOfMonth = remember {
        mutableStateListOf<String>(
            "Jan",
            "Feb",
            "Mar",
            "Apr",
            "May",
            "Jun",
            "Jul",
            "Aug",
            "Sep",
            "Oct",
            "Nov",
            "Dec",
        )
    }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {
        Column(modifier = modifier.padding(32.dp)) {

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .fillMaxWidth()
                    .padding(bottom = 24.dp),
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                TextComponent(
                    text = "Month",
                    fontSize = 22.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surface
                )

                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = "Close Button",
                    modifier = modifier
                        .clip(CircleShape)
                        .clickable(
                            indication = null,
                            interactionSource = interaction,
                            onClick = onDismissRequest
                        )
                        .size(20.dp)
                        .padding(4.dp),
                    tint = MaterialTheme.colorScheme.inversePrimary
                )
            }

            TextComponent(
                text = currentMonth.month.getDisplayName(
                    java.time.format.TextStyle.FULL,
                    Locale.getDefault()
                ),
                fontSize = 22.sp,
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                color = MaterialTheme.colorScheme.surface,
                modifier = Modifier
                    .align(Alignment.CenterHorizontally)
                    .clickable(
                        onClick = {},
                        interactionSource = interaction,
                        indication = null
                    )
            )

            LazyVerticalGrid(
                modifier = modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceBetween,
                columns = GridCells.Fixed(4),
                verticalArrangement = Arrangement.spacedBy(10.dp),
                contentPadding = PaddingValues(vertical = 17.dp)
            ) {
                itemsIndexed(listOfMonth) { index, month ->
                    MonthItemRow(
                        onClick = {
                            selectIndex = index
                            onConfirmation.invoke(index + 1)
                            onDismissRequest.invoke()
                        },
                        name = month,
                        isSelected = index == selectIndex
                    )
                }
            }
        }
    }
}

@Composable
fun MonthItemRow(
    isSelected: Boolean = false,
    name: String,
    onClick: () -> Unit
) {

    val interactionSource = remember { MutableInteractionSource() }

    TextComponent(
        text = name,
        style = TextStyle(
            fontSize = 20.sp,
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
            color = if (isSelected) white else MaterialTheme.colorScheme.surface,
            textAlign = TextAlign.Center
        ),
        modifier = Modifier
            .background(if (isSelected) blue316AFF else transparent, RoundedCornerShape(8.dp))
            .padding(vertical = 5.dp)
            .clickable(
                onClick = onClick,
                interactionSource = interactionSource,
                indication = null
            )
    )

}