package com.ubsidifinance.dailog

import android.os.Build
import androidx.annotation.RequiresApi
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.itemsIndexed
import androidx.compose.foundation.pager.HorizontalPager
import androidx.compose.foundation.pager.rememberPagerState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.IndicatorComponent
import com.ubsidifinance.component.TextComponent
import kotlinx.coroutines.launch
import java.time.YearMonth

@OptIn(ExperimentalMaterial3Api::class)
@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun YearPickerDialog(
    modifier: Modifier = Modifier,
    startYear: YearMonth = YearMonth.now(),
    endYear: YearMonth = YearMonth.now(),
    currentYear: YearMonth = YearMonth.now(),
    onDismissRequest: () -> Unit,
    onConfirmation: (Int) -> Unit,
) {

    val interaction = remember { MutableInteractionSource() }
    val listOfYear = remember {
        val months = generateSequence(startYear) { it.plusMonths(1) }
            .takeWhile { it <= endYear }
            .map { it.year }
            .distinct()
            .toList()
            .reversed()

        mutableStateListOf(*months.toTypedArray())
    }

    var selectIndex by remember { mutableIntStateOf(listOfYear.indexOf(currentYear.year)) }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {
        Column(modifier = modifier.padding(32.dp)) {

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .fillMaxWidth()
                    .padding(bottom = 24.dp),
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                TextComponent(
                    text = "Year",
                    fontSize = 22.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surface
                )

                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = "Close Button",
                    modifier = modifier
                        .clip(CircleShape)
                        .clickable(
                            indication = null,
                            interactionSource = interaction,
                            onClick = onDismissRequest
                        )
                        .size(20.dp)
                        .padding(4.dp),
                    tint = MaterialTheme.colorScheme.inversePrimary
                )
            }



            YearGridPager(
                listOfYear = listOfYear,
                selectIndex = selectIndex,
                currentSelectMonth = currentYear
            ) {
                selectIndex = it
                onConfirmation.invoke(listOfYear[selectIndex])
                onDismissRequest.invoke()
            }

        }
    }
}

@RequiresApi(Build.VERSION_CODES.O)
@OptIn(ExperimentalFoundationApi::class)
@Composable
fun YearGridPager(
    listOfYear: List<Int>,
    selectIndex: Int,
    currentSelectMonth: YearMonth,
    onYearSelected: (Int) -> Unit
) {
    val itemsPerPage = 3 * 4
    val chunkedYears = listOfYear.chunked(itemsPerPage)

    val pagerState = rememberPagerState(pageCount = { chunkedYears.size })
    val interaction = remember { MutableInteractionSource() }
    val scope = rememberCoroutineScope()

    Column {

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .padding(bottom = 18.dp),
            horizontalArrangement = Arrangement.SpaceEvenly
        ) {

            Icon(
                painter = painterResource(R.drawable.ic_back_arrow),
                contentDescription = null,
                modifier = Modifier
                    .clip(CircleShape)
                    .clickable(
                        indication = null,
                        interactionSource = interaction,
                        onClick = {
                            scope.launch {
                                pagerState.animateScrollToPage(pagerState.currentPage - 1)
                            }
                        })
                    .size(24.dp)
                    .padding(5.dp),
                tint = MaterialTheme.colorScheme.inversePrimary
            )

            TextComponent(
                text = currentSelectMonth.year.toString(),
                fontSize = 22.sp,
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                color = MaterialTheme.colorScheme.surface,
                modifier = Modifier
            )


            Icon(
                painter = painterResource(R.drawable.ic_back_arrow),
                contentDescription = null,
                modifier = Modifier
                    .rotate(180f)
                    .clip(CircleShape)
                    .clickable(indication = null, interactionSource = interaction, onClick = {
                        scope.launch {
                            pagerState.animateScrollToPage(pagerState.currentPage + 1)
                        }
                    })
                    .size(24.dp)
                    .padding(5.dp),
                tint = MaterialTheme.colorScheme.inversePrimary
            )

        }

        HorizontalPager(
            state = pagerState
        ) { page ->
            val years = chunkedYears[page]

            LazyVerticalGrid(
                modifier = Modifier.fillMaxWidth(),
                columns = GridCells.Fixed(4),
                verticalArrangement = Arrangement.spacedBy(10.dp),
                horizontalArrangement = Arrangement.SpaceBetween,
                contentPadding = PaddingValues(vertical = 17.dp)
            ) {
                itemsIndexed(years, key = { i, year -> i }) { index, year ->
                    MonthItemRow(
                        onClick = {
                            val realIndex = page * itemsPerPage + index
                            onYearSelected(realIndex)
                        },
                        name = year.toString(),
                        isSelected = (page * itemsPerPage + index) == selectIndex
                    )
                }
            }
        }

        IndicatorComponent(
            totalSize = chunkedYears.size,
            selectedIndex = pagerState.currentPage,
            selectColor = MaterialTheme.colorScheme.inversePrimary,
            modifier = Modifier
                .padding(top = 10.dp)
                .align(Alignment.CenterHorizontally)
        )
    }

}
