package com.ubsidifinance.ui.card

import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.snapping.SnapPosition
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.pager.HorizontalPager
import androidx.compose.foundation.pager.PageSize
import androidx.compose.foundation.pager.rememberPagerState
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalClipboardManager
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.fragment.app.FragmentActivity
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import com.ubsidifinance.R
import com.ubsidifinance.component.IndicatorComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.dailog.RowMenuItem
import com.ubsidifinance.dailog.SnackBarPopup
import com.ubsidifinance.model.CardTabUiState
import com.ubsidifinance.model.OnboardingModel
import com.ubsidifinance.navigation.CardHistory
import com.ubsidifinance.navigation.CardSetting
import com.ubsidifinance.ui.pin.AuthenticationDialog
import com.ubsidifinance.ui.theme.blue316AFF
import com.ubsidifinance.ui.theme.grey_646B7C
import com.ubsidifinance.utils.isResume
import com.ubsidifinance.utils.showToast

@Composable
fun CardScreen(
    modifier: Modifier = Modifier,
    navController: NavController,
    oldNavController: NavController,
    viewmodel: CardViewmodel = hiltViewModel()
) {

    val state = viewmodel.uiEvent.value
    var message by remember { mutableStateOf<String?>(null) }
    var isAuthenticateDialog by remember { mutableStateOf(false) }
    var isHideCardInfo by remember { mutableStateOf(false) }
    val pagerState = rememberPagerState(initialPage = 0, pageCount = {
        state.listOfCard.size
    })
    val scrollState = rememberScrollState()
    val context = LocalContext.current as FragmentActivity

    message?.let {
        SnackBarPopup(
            message = it,
            onDismiss = {
                message = null
            })
    }

    if (isAuthenticateDialog) {
        AuthenticationDialog(
            onDismissRequest = {
                isAuthenticateDialog = false
            },
            fragmentActivity = context,
            onConfirmRequest = {
                viewmodel.onEvent(CardTabUiState.OnCardUnLock(state.listOfCard[pagerState.currentPage]))
            })
    }

    Column {
        Row(
            verticalAlignment = Alignment.CenterVertically,
            modifier = modifier.padding(vertical = 12.dp, horizontal = 24.dp)
        ) {

            TextComponent(
                text = "Cards",
                fontSize = 24.sp,
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                color = MaterialTheme.colorScheme.inversePrimary,
                modifier = modifier
                    .padding(start = 12.dp)
                    .weight(1f)
            )

            Icon(
                painter = painterResource(R.drawable.ic_plus),
                contentDescription = "Add Card",
                modifier = modifier
                    .size(24.dp)
                    .padding(5.dp),
                tint = MaterialTheme.colorScheme.primary
            )

        }

        Column(
            modifier
                .weight(1f)
                .verticalScroll(scrollState)
                .padding(horizontal = 24.dp)
                .padding(bottom = 24.dp)
        ) {

            HorizontalPager(
                state = pagerState,
                pageSpacing = 10.dp,
                pageSize = PageSize.Fixed(290.dp),
                snapPosition = SnapPosition.Center,
                modifier = Modifier
                    .align(Alignment.CenterHorizontally)
                    .padding(top = 20.dp),
            ) { index ->
                Box(
                    modifier
                        .fillMaxWidth()
                ) {
                    CardTabDetailsRowItem(
                        model = state.listOfCard[index],
                        onUnlockCard = {
                            isAuthenticateDialog = true
                        },
                        onPinViewChange = {
                            viewmodel.onEvent(
                                CardTabUiState.OnPinViewChange(
                                    false,
                                    state.listOfCard[index]
                                )
                            )
                        }, onComplete = {
                            viewmodel.onEvent(
                                CardTabUiState.OnPinViewChange(
                                    false,
                                    state.listOfCard[index]
                                )
                            )
                        })
                }

            }

            IndicatorComponent(
                modifier = modifier
                    .fillMaxWidth()
                    .padding(top = 20.dp),
                totalSize = pagerState.pageCount,
                selectedIndex = pagerState.currentPage,
                selectColor = MaterialTheme.colorScheme.inverseSurface
            )

            Row(
                horizontalArrangement = Arrangement.SpaceBetween,
                modifier = modifier
                    .fillMaxWidth()
                    .padding(top = 30.dp)
            ) {
                RowMenuItem(
                    model = OnboardingModel(
                        descriptions = "Lock Card",
                        image = R.drawable.ic_lock
                    )
                ) {
                    message = "Card locked successfully."
                    viewmodel.onEvent(CardTabUiState.OnCardLock(state.listOfCard[pagerState.currentPage]))
                }

                RowMenuItem(
                    model = OnboardingModel(
                        descriptions = "View PIN",
                        image = R.drawable.ic_view_pin
                    )
                ) {
                    viewmodel.onEvent(
                        CardTabUiState.OnPinViewChange(
                            true,
                            state.listOfCard[pagerState.currentPage]
                        )
                    )
                }
                RowMenuItem(
                    model = OnboardingModel(
                        descriptions = "Card Controls",
                        image = R.drawable.ic_setting
                    )
                ) {
                    if (oldNavController.isResume()) {
                        oldNavController.navigate(CardSetting)
                    }
                }
                RowMenuItem(
                    model = OnboardingModel(
                        descriptions = "Lost/Stolen",
                        image = R.drawable.ic_shield_error
                    )
                ) {
                    if (oldNavController.isResume()) {
//                        oldNavController.navigate(CardHistory)
                    }
                }
            }

            Row(
                horizontalArrangement = Arrangement.SpaceBetween,
                modifier = modifier
                    .fillMaxWidth()
                    .padding(top = 30.dp, bottom = 8.dp)
            ) {
                TextComponent(
                    text = "Card Informations",
                    fontSize = 18.sp,
                    fontFamily = FontFamily(Font(R.font.inter_bold, FontWeight.W700)),
                    color = MaterialTheme.colorScheme.surfaceDim,
                    modifier = modifier
                        .weight(1f)
                )

                TextComponent(
                    text = if (isHideCardInfo) "Show info" else "Hide info",
                    fontSize = 12.sp,
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    color = MaterialTheme.colorScheme.surfaceDim,
                    modifier = modifier.clickable(onClick = {
                        isHideCardInfo = !isHideCardInfo
                    })
                )
            }

            RowCardInfo(
                text = if (isHideCardInfo) "**** **** **** 5342" else "5531 2931 3020 5342",
                title = "Card Number",
                isHideInfo = isHideCardInfo
            )

            RowCardInfo(
                text = "02/27",
                title = "Expiration",
                isHideInfo = isHideCardInfo
            )

            RowCardInfo(
                text = if (isHideCardInfo) "***" else "123",
                title = "CVV",
                isHideInfo = isHideCardInfo
            )
        }
    }

}

@Composable
fun RowCardInfo(
    modifier: Modifier = Modifier,
    title: String,
    text: String,
    isHideInfo: Boolean = false
) {

    val interactionSource = remember { MutableInteractionSource() }
    val clipboardManager = LocalClipboardManager.current
    val context = LocalContext.current

    Row(
        modifier = modifier.padding(vertical = 8.dp),
        verticalAlignment = Alignment.CenterVertically
    ) {
        TextComponent(
            text = title,
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
            modifier = modifier.weight(1f),
            fontSize = 12.sp,
            color = grey_646B7C
        )

        TextComponent(
            text = text,
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
            modifier = modifier.padding(end = 11.dp),
            fontSize = 12.sp,
            color = MaterialTheme.colorScheme.surfaceDim
        )

        if (!isHideInfo) {
            Icon(
                modifier = Modifier
                    .size(18.dp)
                    .clickable(interactionSource = interactionSource, indication = null, onClick = {
                        clipboardManager.setText(AnnotatedString(text))
                        context.showToast("Copied to clipboard")
                    }),
                painter = painterResource(R.drawable.ic_copy),
                contentDescription = "",
                tint = blue316AFF
            )
        }
    }

}