package com.ubsidifinance.ui.card_setting

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.MenuAnchorType
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import com.ubsidifinance.R
import com.ubsidifinance.component.ActionBarWithBackButton
import com.ubsidifinance.component.SwitchComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.model.state.CardSettingUiState
import com.ubsidifinance.navigation.BillingInfo
import com.ubsidifinance.navigation.CardReport
import com.ubsidifinance.navigation.ContactInfo
import com.ubsidifinance.ui.theme.black2C2C2C
import com.ubsidifinance.ui.theme.grey_646B7C
import com.ubsidifinance.ui.theme.grey_F9F9F9
import com.ubsidifinance.utils.isResume
import com.ubsidifinance.utils.isValidateEmpty

@Composable
fun CardSettingScreen(
    modifier: Modifier = Modifier,
    navController: NavController,
    viewmodel: CardSettingViewmodel = hiltViewModel()
) {

    val scrollSate = rememberScrollState()
    val state = viewmodel.uiEvent.value
    val prefixText: (@Composable () -> Unit)? =
        {
            TextComponent(
                text = "£",
                fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                fontSize = 14.sp,
                color = black2C2C2C
            )
        }

    Scaffold(
        modifier = modifier.padding(horizontal = 24.dp),
        topBar = {
            ActionBarWithBackButton(
                onBack = {
                    if (navController.isResume()) {
                        navController.popBackStack()
                    }
                },
                title = "Card Controls"
            )
        }) { innerPadding ->
        Column(
            modifier
                .padding(innerPadding)
                .fillMaxSize()
                .verticalScroll(scrollSate)
                .padding(vertical = 24.dp)
        ) {

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .clickable(onClick = { })
                    .padding(bottom = 10.dp)
            ) {
                Image(
                    painter = painterResource(R.drawable.ic_coins),
                    contentDescription = "",
                    modifier = modifier
                        .background(grey_F9F9F9, CircleShape)
                        .size(40.dp)
                        .padding(10.dp)
                )

                Column(
                    modifier = modifier
                        .padding(start = 12.dp, end = 8.dp)
                        .weight(1f)
                ) {
                    TextComponent(
                        text = "Online Limit",
                        fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400)),
                        modifier = modifier.padding(bottom = 4.dp),
                        fontSize = 12.sp,
                        color = grey_646B7C
                    )

                    TextComponent(
                        text = if (state.limitAmount.isValidateEmpty() || !state.isOnlineLimit) "No Limit" else "£" + state.limitAmount + " " + state.limitDuration,
                        fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
                        modifier = modifier,
                        fontSize = 16.sp,
                        color = MaterialTheme.colorScheme.inversePrimary
                    )
                }

                SwitchComponent(
                    onCheckChange = {
                        viewmodel.onEvent(CardSettingUiState.OnOnlineLimitChange(it))
                    },
                    checked = state.isOnlineLimit
                )

            }

            if (state.isOnlineLimit) {
                Row(modifier = modifier.padding(bottom = 20.dp)) {
                    Box(
                        modifier = modifier
                            .weight(1f)
                            .padding(end = 13.dp)
                    ) {
                        TextFieldComponent(
                            value = state.limitAmount,
                            border = false,
                            placeholder = "Enter Amount",
                            textStyle = MaterialTheme.typography.bodySmall.copy(color = black2C2C2C),
                            backgroundColor = grey_F9F9F9,
                            newModifier = modifier,
                            keyboardOptions = KeyboardOptions(
                                imeAction = ImeAction.Done,
                                keyboardType = KeyboardType.Decimal
                            ),
                            prefix = prefixText,
                            onValueChange = {
                                if (it.length <= 5) {
                                    viewmodel.onEvent(CardSettingUiState.OnOnlineLimitTextChange(it))
                                }
                            })
                    }

                    Box(
                        modifier = modifier
                            .weight(1f)
                            .padding(start = 13.dp)
                    ) {
                        LimitDropdown(
                            selectedType = state.limitDuration,
                            onValueChange = {
                                viewmodel.onEvent(CardSettingUiState.OnOnlineLimitDurationChange(it))
                            }
                        )
                    }
                }
            }

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .clickable(onClick = { })
                    .padding(bottom = 10.dp)
            ) {
                Image(
                    painter = painterResource(R.drawable.ic_gambling_menu),
                    contentDescription = "",
                    modifier = modifier
                        .background(grey_F9F9F9, CircleShape)
                        .size(40.dp)
                        .padding(13.dp)
                )

                Column(
                    modifier = modifier
                        .padding(start = 12.dp, end = 8.dp)
                        .weight(1f)
                ) {
                    TextComponent(
                        text = "Gambling Payments",
                        fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400)),
                        modifier = modifier.padding(bottom = 4.dp),
                        fontSize = 12.sp,
                        color = grey_646B7C
                    )

                    TextComponent(
                        text = if (state.isGambling) "Blocked" else "Unblocked",
                        fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
                        modifier = modifier,
                        fontSize = 16.sp,
                        color = MaterialTheme.colorScheme.inversePrimary
                    )
                }
                SwitchComponent(
                    onCheckChange = {
                        viewmodel.onEvent(CardSettingUiState.OnGamblingChange(it))
                    },
                    checked = state.isGambling
                )
            }

            Row(
                verticalAlignment = Alignment.CenterVertically,
                modifier = modifier
                    .clickable(onClick = { })
                    .padding(bottom = 12.dp, top = 5.dp)
            ) {
                Image(
                    painter = painterResource(R.drawable.ic_international),
                    contentDescription = "",
                    modifier = modifier
                        .background(grey_F9F9F9, CircleShape)
                        .size(40.dp)
                        .padding(11.dp)
                )

                TextComponent(
                    text = "International Payment",
                    fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
                    modifier = modifier
                        .padding(start = 12.dp, end = 8.dp)
                        .weight(1f),
                    fontSize = 16.sp,
                    color = MaterialTheme.colorScheme.inversePrimary
                )

                SwitchComponent(
                    onCheckChange = {
                        viewmodel.onEvent(CardSettingUiState.OnInternationalPaymentChange(it))
                    },
                    checked = state.isInternationalPayment
                )
            }

            RowCardSetting(
                onClick = {
                    if (navController.isResume()) {
                        navController.navigate(BillingInfo)
                    }
                },
                heading = "Billing Information",
                imageId = R.drawable.ic_paper
            )

            RowCardSetting(
                onClick = {
                    if (navController.isResume()) {
                        navController.navigate(ContactInfo)
                    }
                },
                heading = "Contact Information",
                imageId = R.drawable.ic_contact
            )

            RowCardSetting(
                onClick = {
                    if (navController.isResume()) {
                        navController.navigate(CardReport)
                    }
                },
                heading = "Card Report",
                imageId = R.drawable.ic_graph
            )

            RowCardSetting(
                onClick = {},
                heading = "Report a Problem",
                imageId = R.drawable.ic_report
            )

            RowCardSetting(
                onClick = {},
                heading = "Delete Card",
                imageId = R.drawable.ic_delete_card
            )
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun LimitDropdown(
    selectedType: String = "", onValueChange: (String) -> Unit
) {
    var expanded by remember { mutableStateOf(false) }

    val accountTypes =
        listOf("Every Spend", "Every Day", "Every Week", "Every Month", "Every Year", "All Time")

    ExposedDropdownMenuBox(
        expanded = expanded,
        onExpandedChange = { expanded = !expanded }
    ) {

        TextFieldComponent(
            newModifier = Modifier.menuAnchor(MenuAnchorType.SecondaryEditable, true),
            value = selectedType,
            textStyle = MaterialTheme.typography.bodySmall.copy(color = black2C2C2C),
            border = false,
            backgroundColor = grey_F9F9F9,
            suffix = {
                Icon(
                    painter = painterResource(R.drawable.ic_down_arrow),
                    contentDescription = null,
                    tint = black2C2C2C,
                    modifier = Modifier
                        .padding(start = 4.dp, end = 0.dp)
                        .size(14.dp)
                        .padding(2.dp)
                        .rotate(if (expanded) 180f else 0f)
                )
            },
            readOnly = true
        ) { }

        ExposedDropdownMenu(
            containerColor = grey_F9F9F9,
            expanded = expanded,
            onDismissRequest = { expanded = false }
        ) {
            accountTypes.forEach { type ->
                TextComponent(
                    color = black2C2C2C,
                    text = type,
                    fontSize = 14.sp,
                    modifier = Modifier
                        .fillMaxWidth()
                        .clickable(
                            onClick = {
                                onValueChange.invoke(type)
                                expanded = false
                            })
                        .padding(horizontal = 12.dp, vertical = 7.dp),
                    fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600))
                )
            }
        }
    }
}


@Composable
fun RowCardSetting(
    modifier: Modifier = Modifier,
    imageId: Int,
    heading: String,
    onClick: () -> Unit

) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .clickable(onClick = onClick)
            .padding(vertical = 12.dp)
    ) {
        Image(
            painter = painterResource(imageId),
            contentDescription = "",
            modifier = modifier
                .background(grey_F9F9F9, CircleShape)
                .size(40.dp)
                .padding(10.dp)
        )

        TextComponent(
            text = heading,
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
            modifier = modifier
                .padding(start = 12.dp, end = 8.dp)
                .weight(1f),
            fontSize = 16.sp,
            color = MaterialTheme.colorScheme.inversePrimary
        )
    }
}