package com.ubsidifinance.ui.category

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewmodel.compose.LocalViewModelStoreOwner
import androidx.lifecycle.viewmodel.compose.viewModel
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.HorizontalDividerComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.model.OnboardingModel
import com.ubsidifinance.model.state.CategoryUiState
import com.ubsidifinance.ui.theme.grey_F9F9F9
import com.ubsidifinance.utils.isValidateEmpty
import java.util.UUID

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CategoryDialog(
    modifier: Modifier = Modifier,
    onDismissRequest: () -> Unit,
    onConfirmation: (OnboardingModel) -> Unit,
) {

    val viewModelStoreOwner = LocalViewModelStoreOwner.current!!
    val uniqueKey = remember { UUID.randomUUID().toString() }
    val viewmodel: CategoryViewmodel = viewModel(
        viewModelStoreOwner = viewModelStoreOwner,
        key = uniqueKey,
        factory = ViewModelProvider.NewInstanceFactory()
    )
    val state = viewmodel.uiEvent.value

    val suffixSearchIcon: (@Composable () -> Unit)? =
        if (state.searchText.isValidateEmpty()) null else {
            {
                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = null,
                    tint = MaterialTheme.colorScheme.secondary,
                    modifier = modifier
                        .size(10.dp)
                        .clickable {
                            viewmodel.onEvent(CategoryUiState.OnSearchChangeValue(""))
                        }
                )
            }
        }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {

        Column(
            modifier = modifier
                .padding(24.dp)
                .fillMaxWidth()
        ) {

            HorizontalDividerComponent(
                thickness = 6.dp, modifier = modifier
                    .width(50.dp)
                    .clip(CircleShape)
                    .align(Alignment.CenterHorizontally)
            )

            TextComponent(
                text = "Choose Category",
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                modifier = modifier.padding(vertical = 24.dp),
                fontSize = 18.sp,
                color = MaterialTheme.colorScheme.surfaceDim
            )

            TextFieldComponent(
                prefix = {
                    Icon(
                        painter = painterResource(R.drawable.ic_search),
                        contentDescription = null,
                        tint = MaterialTheme.colorScheme.secondary,
                        modifier = modifier.size(20.dp)
                    )
                },
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Search),
                suffix = suffixSearchIcon,
                modifier = modifier,
                value = state.searchText,
                placeholder = "Search",
                onValueChange = {
                    viewmodel.onEvent(CategoryUiState.OnSearchChangeValue(it))
                }
            )

            Spacer(modifier.height(24.dp))

            LazyColumn(modifier.fillMaxWidth()) {
                items(state.listOfCategory, key = { item -> item.id }) { item ->
                    CategoryRowItem(model = item) {
                        onDismissRequest.invoke()
                        onConfirmation.invoke(item)
                    }
                }
            }

        }
    }
}

@Composable
fun CategoryRowItem(
    modifier: Modifier = Modifier,
    model: OnboardingModel,
    onClick: () -> Unit
) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .clickable(onClick = onClick)
            .padding(vertical = 12.dp)
    ) {
        Image(
            painter = painterResource(model.image),
            contentDescription = "",
            modifier = modifier
                .background(grey_F9F9F9, CircleShape)
                .size(40.dp)
                .padding(10.dp)
        )

        TextComponent(
            text = model.title,
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
            modifier = modifier
                .padding(start = 12.dp, end = 8.dp)
                .weight(1f),
            fontSize = 14.sp,
            color = MaterialTheme.colorScheme.inversePrimary
        )


    }

}