package com.ubsidifinance.ui.login

import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.text.input.VisualTransformation
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.ubsidifinance.R
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.LabelWithTextField
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.dailog.CustomAlertDialog
import com.ubsidifinance.dailog.LoadingDialog
import com.ubsidifinance.model.state.LoginScreenState
import com.ubsidifinance.model.state.LoginUiState
import com.ubsidifinance.navigation.Screens
import com.ubsidifinance.ui.forgot_password.ForgotPasswordDialog
import com.ubsidifinance.ui.theme.black1E1E1E
import com.ubsidifinance.utils.isResume
import kotlinx.coroutines.flow.collectLatest

@Composable
fun LoginScreen(
    modifier: Modifier = Modifier,
    navController: NavHostController,
    viewmodel: LoginViewmodel = hiltViewModel()
) {

    val state = viewmodel.uiEvent.value

    LaunchedEffect(true) {
        viewmodel.state.collectLatest {
            when (it) {
                is LoginScreenState.NavigateToNextScreen -> {
                    navController.navigate(Screens.Pin.name) {
                        popUpTo(Screens.Login.name) {
                            inclusive = true
                        }
                    }
                }
            }
        }
    }

    if (state.isError) {
        CustomAlertDialog(
            title = state.title,
            text = state.error,
            onDismissRequest = {
                viewmodel.onEvent(LoginUiState.OnError(isError = false))
            }, onConfirmation = {

            })
    }

    if (state.isLoading) {
        LoadingDialog()
    }

    if (state.isForgotPassword) {
        ForgotPasswordDialog(
            onDismissRequest = {
                viewmodel.onEvent(LoginUiState.OnForgotPasswordDialog(false))
            },
            onConfirmation = {
                viewmodel.onEvent(
                    LoginUiState.OnError(
                        isError = true,
                        error = "A link to reset your password has been sent to the registered email.",
                        title = "Reset Link Sent"
                    )
                )
            })
    }

    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(horizontal = 24.dp)
            .padding(bottom = 24.dp)
    ) {

        Image(
            painter = painterResource(R.drawable.img_logo_with_ubsidi),
            contentDescription = "Logo",
            contentScale = ContentScale.FillHeight,
            modifier = modifier
                .align(Alignment.CenterHorizontally)
                .padding(top = 30.dp)
                .height(83.dp)
        )

        Column(modifier = modifier.weight(1f), verticalArrangement = Arrangement.Center) {

            LabelWithTextField(
                label = "Email or Phone Number",
                placeHolder = "Enter Your Email",
                value = state.email,
                onValueChange = {
                    viewmodel.onEvent(LoginUiState.OnEmailChange(it))
                }
            )

            LabelWithTextField(
                label = "Password",
                visualTransformation = if (state.isVisiblePassword) VisualTransformation.None else PasswordVisualTransformation(),
                placeHolder = "Enter Your Password",
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Password,
                    imeAction = ImeAction.Done
                ),
                value = state.password,
                suffix = {
                    Icon(
                        painter = painterResource(if (!state.isVisiblePassword) R.drawable.ic_hide_password else R.drawable.ic_show_password),
                        contentDescription = null,
                        tint = MaterialTheme.colorScheme.secondary,
                        modifier = modifier
                            .clickable {
                                viewmodel.onEvent(LoginUiState.OnPasswordVisibilityChange)
                            }
                    )
                },
                modifier = modifier.padding(top = 32.dp),
                onValueChange = {
                    viewmodel.onEvent(LoginUiState.OnPasswordChange(it))
                }
            )

            TextComponent(
                text = "Forgot password?",
                fontSize = 14.sp,
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                color = MaterialTheme.colorScheme.primary,
                modifier = modifier
                    .align(Alignment.End)
                    .padding(top = 8.dp)
                    .clickable {
                        //viewmodel.onEvent(LoginUiState.OnForgotPasswordDialog(true))
                        navController.navigate(Screens.ForgotPassword.name)
                    }
            )

            ButtonComponent(
                text = "Login",
                onClick = {
                    viewmodel.onEvent(LoginUiState.ClickOnSubmit)
                },
                modifier = modifier.padding(top = 32.dp)
            )

        }

        TextComponent(
            text = "Don’t have an account? Register now",
            modifier = modifier
                .align(
                    Alignment.CenterHorizontally
                ),
            fontFamily = FontFamily(
                Font(R.font.inter_regular, FontWeight.W400),
            ),
            fontSize = 15.sp,
            color = MaterialTheme.colorScheme.surface
        )

        ButtonComponent(
            text = "Register",
            modifier = modifier.padding(top = 10.dp),
            onClick = {
                if (navController.isResume()) {
                    navController.navigate(Screens.RegisterCommit.name)
                }
            },
            colors = ButtonDefaults.buttonColors(
                containerColor = MaterialTheme.colorScheme.surfaceBright,
                contentColor = black1E1E1E,
                disabledContentColor = Color.White,
                disabledContainerColor = MaterialTheme.colorScheme.primary.copy(alpha = 0.5f)
            )
        )
    }
}