package com.ubsidifinance.ui.login

import androidx.compose.runtime.mutableStateOf
import androidx.core.text.isDigitsOnly
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.ubsidifinance.base.ApiResult
import com.ubsidifinance.model.state.LoginScreenState
import com.ubsidifinance.model.state.LoginState
import com.ubsidifinance.model.state.LoginUiState
import com.ubsidifinance.navigation.Screens
import com.ubsidifinance.network.repo.AuthRepo
import com.ubsidifinance.utils.Preferences
import com.ubsidifinance.utils.isValidEmail
import com.ubsidifinance.utils.isValidateEmpty
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class LoginViewmodel @Inject constructor(
    private val repo: AuthRepo,
    private val preferences: Preferences
) : ViewModel() {

    private val _uiState = mutableStateOf(LoginState())
    val uiEvent = _uiState

    private val _state = MutableSharedFlow<LoginScreenState>()
    val state = _state.asSharedFlow()

    fun onEvent(event: LoginUiState) {
        when (event) {
            is LoginUiState.OnEmailChange -> {
                _uiState.value = _uiState.value.copy(email = event.text)
            }

            is LoginUiState.OnPasswordChange -> {
                _uiState.value = _uiState.value.copy(password = event.text)
            }

            is LoginUiState.ClickOnSubmit -> {
                validateField()
            }

            is LoginUiState.OnError -> {
                _uiState.value =
                    _uiState.value.copy(
                        isError = event.isError,
                        error = event.error,
                        title = event.title
                    )
            }

            is LoginUiState.OnForgotPasswordDialog -> {
                _uiState.value =
                    _uiState.value.copy(isForgotPassword = event.isShow)
            }

            LoginUiState.OnPasswordVisibilityChange -> {
                _uiState.value =
                    _uiState.value.copy(isVisiblePassword = !_uiState.value.isVisiblePassword)
            }
        }
    }

    private fun login() {
        val email = _uiState.value.email
        val password = _uiState.value.password
        viewModelScope.launch {
            repo.executeLogin(email, password).collect { response ->
                when (response) {
                    is ApiResult.Error -> {
                        _uiState.value =
                            _uiState.value.copy(
                                isLoading = false,
                                isError = true,
                                error = response.message,
                                title = "Error"
                            )
                    }

                    ApiResult.Loading -> {
                        _uiState.value =
                            _uiState.value.copy(
                                isLoading = true,
                            )
                    }

                    is ApiResult.Success -> {
                        preferences.loginUserModel = response.data
                        viewModelScope.launch {
                            _state.emit(LoginScreenState.NavigateToNextScreen(Screens.Pin.name))
                        }
                        _uiState.value =
                            _uiState.value.copy(
                                isLoading = false,
                            )
                    }
                }
            }
        }
    }

    private fun validateField() {
        val email = _uiState.value.email
        val password = _uiState.value.password

        when {

            email.isValidateEmpty() -> {
                onEvent(LoginUiState.OnError(isError = true, error = "Please enter email/mobile"))
            }

            email.isDigitsOnly() && email.length !in 9..13 -> {
                onEvent(
                    LoginUiState.OnError(
                        isError = true,
                        error = "Please enter valid phone number"
                    )
                )
            }

            !email.isDigitsOnly() && !email.isValidEmail() -> {
                onEvent(
                    LoginUiState.OnError(
                        isError = true,
                        error = "Please enter valid email address"
                    )
                )
            }

            password.isValidateEmpty() -> {
                onEvent(LoginUiState.OnError(isError = true, error = "Please enter password"))
            }

            else -> {
                login()
//                viewModelScope.launch {
//                    _state.emit(LoginScreenState.NavigateToNextScreen(Screens.Pin.name))
//                }
            }
        }
    }
}