package com.ubsidifinance.ui.new_recipient

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.MenuAnchorType
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewmodel.compose.LocalViewModelStoreOwner
import androidx.lifecycle.viewmodel.compose.viewModel
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.HorizontalDividerComponent
import com.ubsidifinance.component.LabelWithTextField
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.model.AccountModel
import com.ubsidifinance.model.state.RecipientUiState
import java.util.UUID

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun NewRecipientScreen(
    modifier: Modifier = Modifier,
    onDismissRequest: () -> Unit,
    onConfirmation: (AccountModel) -> Unit,
) {

    val viewModelStoreOwner = LocalViewModelStoreOwner.current!!
    val uniqueKey = remember { UUID.randomUUID().toString() }
    val viewmodel: RecipientViewmodel = viewModel(
        viewModelStoreOwner = viewModelStoreOwner,
        key = uniqueKey,
        factory = ViewModelProvider.NewInstanceFactory()
    )
    val scrollState = rememberScrollState()
    val state = viewmodel.uiEvent.value

    if (state.isCheckAccount) {
        CheckRecipientScreen(
            onConfirmation = {
                onDismissRequest.invoke()
                onConfirmation.invoke(it)
            },
            viewmodel = viewmodel,
            onDismissRequest = {
                viewmodel.onEvent(RecipientUiState.OnCheckAccountChange(false))
            }
        )
    }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {

        Column(
            modifier = modifier
                .padding(24.dp)
                .fillMaxWidth()
        ) {

            HorizontalDividerComponent(
                thickness = 6.dp, modifier = modifier
                    .width(50.dp)
                    .clip(CircleShape)
                    .align(Alignment.CenterHorizontally)
            )

            Row(
                modifier = modifier
                    .fillMaxWidth()
                    .padding(top = 24.dp),
                verticalAlignment = Alignment.CenterVertically,
            ) {

                Icon(
                    painter = painterResource(R.drawable.ic_back_arrow),
                    contentDescription = null,
                    modifier = Modifier
                        .clip(CircleShape)
                        .clickable(onClick = onDismissRequest)
                        .size(24.dp)
                        .padding(4.dp),
                    tint = MaterialTheme.colorScheme.inversePrimary
                )

                TextComponent(
                    text = "New recipient",
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    modifier = modifier
                        .weight(1f)
                        .padding(horizontal = 12.dp),
                    fontSize = 18.sp,
                    color = MaterialTheme.colorScheme.surfaceDim
                )

                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = null,
                    modifier = Modifier
                        .clip(CircleShape)
                        .clickable(onClick = onDismissRequest)
                        .size(24.dp)
                        .padding(7.dp),
                    tint = MaterialTheme.colorScheme.inversePrimary
                )

            }

            Column(modifier = modifier.verticalScroll(scrollState)) {

                TextComponent(
                    text = "Account Type",
                    fontSize = 14.sp,
                    color = MaterialTheme.colorScheme.secondary,
                    modifier = modifier.padding(top = 24.dp),
                    fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500))
                )
                AccountTypeDropdown(selectedType = state.accountType, onValueChange = {
                    viewmodel.onEvent(RecipientUiState.OnAccountTypeChange(it))
                })

                LabelWithTextField(
                    label = "Full Name",
                    placeHolder = "Enter Full Name",
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Next),
                    modifier = modifier.padding(top = 16.dp),
                    onValueChange = {
                        viewmodel.onEvent(RecipientUiState.OnNameChange(it))
                    },
                    value = state.fullName
                )

                LabelWithTextField(
                    label = "Sort Code",
                    placeHolder = "Enter Sort Code",
                    keyboardOptions = KeyboardOptions(
                        imeAction = ImeAction.Next,
                        keyboardType = KeyboardType.Number
                    ),
                    modifier = modifier.padding(top = 16.dp),
                    onValueChange = {
                        viewmodel.onEvent(RecipientUiState.OnSortCodeChange(it))
                    },
                    value = state.sortCode
                )

                LabelWithTextField(
                    label = "Account Number",
                    placeHolder = "Enter Account Number",
                    keyboardOptions = KeyboardOptions(
                        imeAction = ImeAction.Done,
                        keyboardType = KeyboardType.Number
                    ),
                    modifier = modifier.padding(top = 16.dp),
                    onValueChange = {
                        viewmodel.onEvent(RecipientUiState.OnAccountNoChange(it))
                    },
                    value = state.accountNumber
                )
            }

            ButtonComponent(
                enable = state.isEnableButton,
                modifier = modifier.padding(top = 24.dp),
                text = "Check",
                onClick = {
                    viewmodel.onEvent(RecipientUiState.OnCheckAccountChange(true))
                }
            )
        }
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun AccountTypeDropdown(
    selectedType: String, onValueChange: (String) -> Unit
) {
    var expanded by remember { mutableStateOf(false) }

    val accountTypes =
        listOf("Personal Account", "Savings Account", "Checking Account", "Business Account")

    ExposedDropdownMenuBox(
        expanded = expanded,
        onExpandedChange = { expanded = !expanded }
    ) {

        TextFieldComponent(
            newModifier = Modifier.menuAnchor(MenuAnchorType.SecondaryEditable, true),
            value = selectedType,
            placeholder = "Select Account Type",
            suffix = {
                Icon(
                    painter = painterResource(R.drawable.ic_down_arrow),
                    contentDescription = null,
                    tint = MaterialTheme.colorScheme.onSecondary,
                    modifier = Modifier
                        .padding(start = 4.dp, top = 2.dp, end = 0.dp)
                        .size(16.dp)
                        .padding(2.dp)
                        .rotate(if (expanded) 180f else 0f)
                )
            },
            readOnly = true
        ) { }

        ExposedDropdownMenu(
            containerColor = MaterialTheme.colorScheme.primaryContainer,
            expanded = expanded,
            onDismissRequest = { expanded = false }
        ) {
            accountTypes.forEach { type ->
                DropdownMenuItem(
                    text = {
                        TextComponent(
                            color = MaterialTheme.colorScheme.inversePrimary,
                            text = type,
                            fontSize = 14.sp,
                            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600))
                        )
                    },
                    onClick = {
                        onValueChange.invoke(type)
                        expanded = false
                    }
                )
            }
        }
    }
}
