package com.ubsidifinance.ui.recipient_contact

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.viewmodel.compose.LocalViewModelStoreOwner
import androidx.lifecycle.viewmodel.compose.viewModel
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.HorizontalDividerComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.model.AccountModel
import com.ubsidifinance.model.state.RecipientContactUiState
import com.ubsidifinance.ui.new_recipient.NewRecipientScreen
import com.ubsidifinance.ui.theme.grey_9499A5
import com.ubsidifinance.ui.theme.grey_F9F9F9
import com.ubsidifinance.utils.isValidateEmpty
import com.ubsidifinance.utils.toInitials
import java.util.UUID

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun RecipientContactDialog(
    modifier: Modifier = Modifier,
    onDismissRequest: () -> Unit,
    onConfirmation: (AccountModel) -> Unit,
) {
    val viewModelStoreOwner = LocalViewModelStoreOwner.current!!
    val uniqueKey = remember { UUID.randomUUID().toString() }
    val viewmodel: RecipientContactViewmodel = viewModel(
        viewModelStoreOwner = viewModelStoreOwner,
        key = uniqueKey,
        factory = ViewModelProvider.NewInstanceFactory()
    )

    val state = viewmodel.uiEvent.value
    var showNewRecipient by remember { mutableStateOf(false) }

    if (showNewRecipient) {
        NewRecipientScreen(
            onConfirmation = {
                onDismissRequest.invoke()
                onConfirmation.invoke(it)
            },
            onDismissRequest = {
                showNewRecipient = false
            }
        )
    }

    val suffixSearchIcon: (@Composable () -> Unit)? =
        if (state.searchText.isValidateEmpty()) null else {
            {
                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = null,
                    tint = MaterialTheme.colorScheme.secondary,
                    modifier = modifier
                        .size(10.dp)
                        .clickable {
                            viewmodel.onEvent(RecipientContactUiState.OnSearchChangeValue(""))
                        }
                )
            }
        }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {

        Column(
            modifier = modifier
                .padding(24.dp)
                .fillMaxWidth()
        ) {

            HorizontalDividerComponent(
                thickness = 6.dp, modifier = modifier
                    .width(50.dp)
                    .clip(CircleShape)
                    .align(Alignment.CenterHorizontally)
            )

            Row(
                modifier = modifier.fillMaxWidth(),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {

                TextComponent(
                    text = "Transfer",
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    modifier = modifier.padding(top = 15.dp),
                    fontSize = 18.sp,
                    color = MaterialTheme.colorScheme.surfaceDim
                )

                TextComponent(
                    text = "New recipient",
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    modifier = modifier
                        .padding(top = 15.dp)
                        .clickable(onClick = {
                            showNewRecipient = true
                        }),
                    fontSize = 14.sp,
                    color = MaterialTheme.colorScheme.primary
                )
            }

            Spacer(modifier = modifier.height(24.dp))

            TextFieldComponent(
                prefix = {
                    Icon(
                        painter = painterResource(R.drawable.ic_search),
                        contentDescription = null,
                        tint = MaterialTheme.colorScheme.secondary,
                        modifier = modifier.size(20.dp)
                    )
                },
                keyboardOptions = KeyboardOptions(imeAction = ImeAction.Search),
                suffix = suffixSearchIcon,
                value = state.searchText,
                placeholder = "Search contact",
                onValueChange = {
                    viewmodel.onEvent(RecipientContactUiState.OnSearchChangeValue(it))
                }
            )

            LazyColumn(
                modifier = modifier
                    .padding(top = 24.dp)
                    .fillMaxWidth()
            ) {

                itemsIndexed(state.listOfCategory, key = { i, item -> i }) { i, item ->
                    RecipientRowItem(
                        model = item,
                        onClick = {
                            onDismissRequest.invoke()
                            onConfirmation.invoke(item)
                        }
                    )
                }

            }
        }
    }
}

@Composable
fun RecipientRowItem(
    modifier: Modifier = Modifier,
    model: AccountModel,
    onClick: () -> Unit
) {


    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .fillMaxWidth()
            .clickable(onClick = onClick)
            .padding(vertical = 10.dp)
    ) {
        TextComponent(
            text = model.fullName.toInitials(),
            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
            modifier = modifier
                .background(grey_F9F9F9, CircleShape)
                .padding(10.dp),
            fontSize = 14.sp,
            color = grey_9499A5
        )

        Column(modifier = modifier.padding(start = 8.dp)) {
            TextComponent(
                text = model.fullName,
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                modifier = modifier,
                fontSize = 14.sp,
                color = MaterialTheme.colorScheme.inversePrimary
            )

            TextComponent(
                text = "SC: ${model.sortCode} AC: ${model.accountNumber}",
                fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
                modifier = modifier
                    .padding(top = 6.dp),
                fontSize = 12.sp,
                color = grey_9499A5
            )
        }
    }
}



