package com.ubsidifinance.ui.register.capture_photo

import android.net.Uri
import androidx.camera.core.ImageCapture
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavHostController
import coil3.compose.rememberAsyncImagePainter
import com.ubsidifinance.R
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.model.RegisterModel
import com.ubsidifinance.ui.theme.black1E1E1E
import com.ubsidifinance.ui.theme.black232323
import com.ubsidifinance.ui.theme.green_009951
import com.ubsidifinance.ui.theme.grey_F5F5F5
import com.ubsidifinance.ui.theme.red_900B09
import com.ubsidifinance.utils.isResume
import com.ubsidifinance.utils.takePictureAndCheckFace
import com.ubsidifinance.utils.toNonNullString
import com.ubsidifinance.utils.verifyDocumentScreen
import java.io.File

@Composable
fun CapturePhotoScreen(
    modifier: Modifier = Modifier,
    navController: NavHostController,
    registerModel: RegisterModel
) {

    var imageCapture by remember { mutableStateOf<ImageCapture?>(null) }
    val capturedImageUri = remember { mutableStateOf<Uri?>(null) }
    var isError by remember { mutableStateOf<Boolean>(false) }
    val context = LocalContext.current

    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(24.dp)
    ) {
        Column(modifier = modifier.weight(1f)) {
            Icon(
                painter = painterResource(R.drawable.ic_back_arrow),
                contentDescription = null,
                modifier = modifier
                    .padding(top = 16.dp)
                    .size(24.dp)
                    .padding(5.dp)
                    .clickable {
                        navController.popBackStack()
                    },
                tint = MaterialTheme.colorScheme.inversePrimary
            )

            TextComponent(
                text = "Facial recognition",
                fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                modifier = modifier.padding(top = 24.dp),
                fontSize = 24.sp,
                color = MaterialTheme.colorScheme.onSecondary
            )

            TextComponent(
                text = "If any information is incorrect, please retake the ID photo.",
                fontSize = 14.sp,
                modifier = modifier.padding(top = 8.dp),
                fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400)),
                color = MaterialTheme.colorScheme.inverseOnSurface
            )

            if (capturedImageUri.value != null) {
                Image(
                    contentDescription = "",
                    painter = rememberAsyncImagePainter(capturedImageUri.value),
                    contentScale = ContentScale.FillBounds,
                    modifier = modifier
                        .padding(top = 48.dp)
                        .fillMaxHeight(0.6f)
                        .clip(CircleShape)
                        .border(
                            1.dp,
                            if (isError) red_900B09 else green_009951,
                            CircleShape
                        )
                )

                if (isError) {
                    TextComponent(
                        text = "The photo is invalid.",
                        fontSize = 14.sp,
                        fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W600)),
                        color = red_900B09,
                        modifier = modifier.padding(top = 20.dp)
                    )
                }
            } else {
                PickImageScreen(
                    modifier = modifier
                        .padding(top = 45.dp)
                        .fillMaxHeight(0.6f)
                        .clip(CircleShape)
                ) {
                    imageCapture = it
                }
            }

        }

        if (isError) {
            ButtonComponent(
                text = "Retake",
                modifier = modifier.padding(bottom = 10.dp),
                onClick = {
                    isError = false
                    capturedImageUri.value = null
                },
                colors = ButtonDefaults.buttonColors(
                    containerColor = MaterialTheme.colorScheme.surfaceBright,
                    contentColor = black1E1E1E,
                    disabledContentColor = Color.White,
                    disabledContainerColor = MaterialTheme.colorScheme.primary.copy(alpha = 0.5f)
                )
            )

        } else {

            Box(modifier = modifier.fillMaxWidth()) {
                if (capturedImageUri.value != null) {

                    Image(
                        painter = painterResource(R.drawable.ic_close),
                        contentDescription = "Retake",
                        modifier = modifier
                            .padding(bottom = 24.dp)
                            .align(Alignment.BottomStart)
                            .background(grey_F5F5F5, CircleShape)
                            .clip(CircleShape)
                            .clickable(onClick = {
                                isError = false
                                capturedImageUri.value = null
                            })
                            .padding(10.dp)
                            .size(18.dp)
                            .padding(2.dp)
                    )

                    Icon(
                        tint = black232323,
                        painter = painterResource(R.drawable.ic_check),
                        contentDescription = "Document Check",
                        modifier = modifier
                            .padding(bottom = 24.dp)
                            .align(Alignment.BottomEnd)
                            .background(grey_F5F5F5, CircleShape)
                            .clip(CircleShape)
                            .clickable(onClick = {
                                if (navController.isResume()) {
                                    val model = registerModel
                                    model.type = verifyDocumentScreen
                                    model.profile = capturedImageUri.value?.path.toNonNullString()
                                    navController.navigate(model)
                                }
                            })
                            .padding(10.dp)
                            .size(18.dp)
                    )
                }

                Image(
                    painter = painterResource(R.drawable.ic_camera),
                    contentDescription = "Take Document Picture",
                    modifier = modifier
                        .padding(bottom = 24.dp)
                        .align(Alignment.BottomCenter)
                        .background(grey_F5F5F5, CircleShape)
                        .clip(CircleShape)
                        .clickable(onClick = {
                            val photoFile = File(
                                context.cacheDir,
                                "Profile_${System.currentTimeMillis()}.jpg"
                            )

                            imageCapture?.takePictureAndCheckFace(
                                context = context,
                                photoFile,
                                onNoFaceDetected = {
                                    isError = true
                                    val uri = Uri.fromFile(photoFile)
                                    capturedImageUri.value = uri
                                },
                                onFaceDetected = {
                                    isError = false
                                    val uri = Uri.fromFile(photoFile)
                                    capturedImageUri.value = uri

                                })
                        })
                        .padding(10.dp)
                        .size(18.dp)
                )
            }
        }
    }

}
