package com.ubsidifinance.ui.register.mobile

import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.consumeWindowInsets
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.BasicTextField
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.ubsidifinance.R
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.HeaderProgress
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.keyboardController
import com.ubsidifinance.dailog.CountryDialog
import com.ubsidifinance.model.RegisterModel
import com.ubsidifinance.model.state.MobileNumberUiState
import com.ubsidifinance.ui.theme.grey_B3B3B3
import com.ubsidifinance.utils.PhoneVisualTransformation
import com.ubsidifinance.utils.getExampleMobileNumber
import com.ubsidifinance.utils.isResume
import com.ubsidifinance.utils.verifyMobileScreen

@Composable
fun PhoneNumberScreen(
    modifier: Modifier = Modifier,
    navController: NavHostController,
    viewmodel: PhoneNumberViewmodel = hiltViewModel()
) {

    val state = viewmodel.uiEvent.value
    val interaction = remember { MutableInteractionSource() }
    val context = LocalContext.current
    val keyboardController = keyboardController()

    if (state.showCountryPicker) {
        CountryDialog(
            viewmodel = viewmodel,
            onDismissRequest = {
                viewmodel.onEvent(MobileNumberUiState.OnShowCountryDialog(false))
            })
    }

    LaunchedEffect(Unit) {
        viewmodel.fetchCountry(context)
    }

    Scaffold { contentPadding ->
        Column(
            modifier = modifier
                .consumeWindowInsets(contentPadding)
                .imePadding()
                .padding(24.dp)
        ) {
            Column(modifier = modifier.weight(1f)) {

                HeaderProgress()

                Icon(
                    painter = painterResource(R.drawable.ic_back_arrow),
                    contentDescription = null,
                    modifier = modifier
                        .padding(top = 16.dp)
                        .size(24.dp)
                        .padding(5.dp)
                        .clickable {
                            if (navController.isResume()) {
                                keyboardController?.hide()
                                navController.popBackStack()
                            }
                        },
                    tint = MaterialTheme.colorScheme.inversePrimary
                )

                TextComponent(
                    text = "Begin now!", fontFamily = FontFamily(
                        Font(
                            R.font.inter_semi_bold,
                            FontWeight.W600
                        ),
                    ),
                    modifier = modifier.padding(top = 24.dp),
                    fontSize = 24.sp,
                    color = MaterialTheme.colorScheme.onSecondary
                )

                TextComponent(
                    text = "Please provide your mobile number to verify your account.",
                    fontSize = 14.sp,
                    modifier = modifier.padding(top = 8.dp),
                    fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400)),
                    color = MaterialTheme.colorScheme.inverseOnSurface
                )

                BasicTextField(
                    value = state.mobileNo,
                    onValueChange = {
                        val newNumber = it.take(
                            getExampleMobileNumber(
                                state.selectedCountry.code,
                                true
                            ).count { it == '0' })
                        viewmodel.onEvent(MobileNumberUiState.OnNumberChange(newNumber))
                    },
                    keyboardActions = KeyboardActions(),
                    keyboardOptions = KeyboardOptions(
                        imeAction = ImeAction.Done,
                        keyboardType = KeyboardType.Phone
                    ),
                    singleLine = true,
                    visualTransformation = PhoneVisualTransformation(
                        getExampleMobileNumber(
                            state.selectedCountry.code,
                            true
                        )
                    ),
                    textStyle = TextStyle(
                        fontSize = 14.sp,
                        color = MaterialTheme.colorScheme.secondary,
                        fontFamily = FontFamily(
                            Font(
                                R.font.inter_medium,
                                FontWeight.W600
                            )
                        )
                    ),
                    modifier = modifier
                        .fillMaxWidth()
                        .padding(top = 24.dp)
                        .border(
                            1.dp, MaterialTheme.colorScheme.surfaceContainerLow,
                            RoundedCornerShape(10.dp)
                        )
                        .padding(horizontal = 12.dp, vertical = 8.dp),
                    decorationBox = { innerTextField ->
                        Row(verticalAlignment = Alignment.CenterVertically) {
                            Row(
                                modifier
                                    .size(44.dp, 24.dp)
                                    .clickable(
                                        interactionSource = interaction,
                                        indication = null,
                                        onClick = {
                                            viewmodel.onEvent(
                                                MobileNumberUiState.OnShowCountryDialog(
                                                    true
                                                )
                                            )
                                        }),
                                verticalAlignment = Alignment.CenterVertically
                            ) {
                                TextComponent(text = state.selectedCountry.flag, fontSize = 24.sp)
                                Icon(
                                    painter = painterResource(R.drawable.ic_down_arrow),
                                    contentDescription = null,
                                    tint = MaterialTheme.colorScheme.onSecondary,
                                    modifier = modifier
                                        .padding(start = 4.dp, top = 2.dp)
                                        .size(16.dp)
                                        .padding(2.dp)
                                        .align(Alignment.CenterVertically)
                                )
                            }

                            TextComponent(
                                text = state.selectedCountry.dial_code,
                                fontSize = 14.sp,
                                fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W600)),
                                modifier = modifier.padding(start = 8.dp, end = 4.dp),
                                color = MaterialTheme.colorScheme.onSecondary
                            )

                            Box(modifier.weight(1f)) {
                                if (state.mobileNo.isEmpty()) {
                                    TextComponent(
                                        text = getExampleMobileNumber(
                                            state.selectedCountry.code,
                                            true
                                        ),
                                        fontSize = 14.sp,
                                        color = grey_B3B3B3,
                                        fontFamily = FontFamily(
                                            Font(
                                                R.font.inter_medium,
                                                FontWeight.W600
                                            )
                                        )
                                    )
                                }
                                innerTextField()
                            }
                        }

                    }
                )
            }

            TextComponent(
                text = "By clicking \"Next,\" you accept the Privacy policy and Terms of service",
                modifier = modifier.align(Alignment.CenterHorizontally),
                style = TextStyle(
                    fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W500)),
                    fontSize = 14.sp,
                    color = MaterialTheme.colorScheme.inverseOnSurface,
                    textAlign = TextAlign.Center
                ),
            )

            ButtonComponent(
                modifier = modifier.padding(top = 24.dp),
                text = "Next",
                onClick = {
                    if (navController.isResume()) {
                        keyboardController?.hide()
                        val model = RegisterModel()
                        model.mobileNo = state.mobileNo
                        model.countryCode = state.selectedCountry.dial_code
                        model.type = verifyMobileScreen
                        navController.navigate(model)
                    }
                })
        }
    }


}