package com.ubsidifinance.ui.transaction

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusRequester
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import com.ubsidifinance.R
import com.ubsidifinance.component.ActionBarWithBackButton
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.model.state.SearchTransactionUiState
import com.ubsidifinance.navigation.TransactionDetail
import com.ubsidifinance.ui.home_page.TransactionItemRow
import com.ubsidifinance.ui.transaction.filter_transaction.TransactionFilterScreen
import com.ubsidifinance.utils.isResume
import com.ubsidifinance.utils.isValidateEmpty
import kotlinx.coroutines.delay

@OptIn(ExperimentalFoundationApi::class)
@Composable
fun SearchTransactionScreen(
    modifier: Modifier = Modifier,
    navController: NavController,
    viewmodel: SearchTransactionViewmodel = hiltViewModel()
) {

    val state = viewmodel.uiEvent.value
    val focusRequester = remember { FocusRequester() }
    val keyboardController = LocalSoftwareKeyboardController.current
    val suffixSearchIcon: (@Composable () -> Unit)? =
        if (state.searchQuery.isValidateEmpty()) null else {
            {
                Icon(
                    painter = painterResource(R.drawable.ic_close),
                    contentDescription = null,
                    tint = MaterialTheme.colorScheme.secondary,
                    modifier = modifier
                        .size(10.dp)
                        .clickable {
                            viewmodel.onEvent(SearchTransactionUiState.OnSearchTextChange(""))
                        }
                )
            }
        }

    LaunchedEffect(Unit) {
        delay(300)
        focusRequester.requestFocus()
        keyboardController?.show()
    }

    Column(modifier = modifier.padding(horizontal = 24.dp)) {
        ActionBarWithBackButton(
            title = "Search Transactions",
            onBack = {
                navController.popBackStack()
            })

        Row(
            modifier = modifier
                .fillMaxWidth()
                .padding(top = 7.dp),
            horizontalArrangement = Arrangement.SpaceBetween,
            verticalAlignment = Alignment.CenterVertically
        ) {
            TextComponent(
                text = "Select Card",
                color = MaterialTheme.colorScheme.onSecondary,
                fontSize = 14.sp,
                fontFamily = FontFamily(
                    Font(R.font.inter_semi_bold, FontWeight.W500)
                )
            )

            Row(verticalAlignment = Alignment.CenterVertically) {
                TextComponent(
                    text = "Jamal Ahmed - 7723",
                    color = MaterialTheme.colorScheme.onSecondary,
                    fontSize = 14.sp,
                    fontFamily = FontFamily(
                        Font(R.font.inter_semi_bold, FontWeight.W500)
                    )
                )

                Icon(
                    painter = painterResource(R.drawable.ic_down_arrow),
                    contentDescription = null,
                    tint = MaterialTheme.colorScheme.onSecondary,
                    modifier = modifier
                        .padding(start = 10.dp, top = 2.dp)
                        .size(16.dp)
                        .padding(2.dp)
                        .align(Alignment.CenterVertically)
                )

            }

        }

        Spacer(modifier.height(20.dp))

        Row(verticalAlignment = Alignment.CenterVertically) {

            Box(modifier = modifier.weight(1f)) {
                TextFieldComponent(
                    focusable = true,
                    focusRequester = focusRequester,
                    prefix = {
                        Icon(
                            painter = painterResource(R.drawable.ic_search),
                            contentDescription = null,
                            tint = MaterialTheme.colorScheme.secondary,
                            modifier = modifier.size(20.dp)
                        )
                    },
                    keyboardOptions = KeyboardOptions(imeAction = ImeAction.Search),
                    suffix = suffixSearchIcon,
                    modifier = modifier,
                    value = state.searchQuery,
                    placeholder = "Search transaction",
                    onValueChange = {
                        viewmodel.onEvent(SearchTransactionUiState.OnSearchTextChange(it))
                    }
                )
            }

            Icon(
                painter = painterResource(R.drawable.ic_filter),
                contentDescription = null,
                tint = MaterialTheme.colorScheme.secondary,
                modifier = modifier
                    .padding(start = 16.dp)
                    .clickable(onClick = {
                        viewmodel.onEvent(SearchTransactionUiState.FilterDialog(true))
                    })
                    .size(24.dp)
            )
        }

        LazyColumn(
            modifier = modifier
                .weight(1f)
                .padding(top = 5.dp)
        ) {
            state.listOfTransaction.forEach {
                stickyHeader(key = it.date) {
                    Column(
                        modifier = modifier
                            .fillMaxWidth()
                            .background(MaterialTheme.colorScheme.background)
                            .padding(vertical = 8.dp)
                    ) {
                        TextComponent(
                            text = it.date,
                            fontSize = 14.sp,
                            color = MaterialTheme.colorScheme.surface,
                            fontFamily = FontFamily(Font(R.font.inter_regular, FontWeight.W400))
                        )
                    }
                }

                items(it.listOfTransaction) { item ->
                    TransactionItemRow(model = item, onClick = {
                        if (navController.isResume()) {
                            navController.navigate(TransactionDetail)
                        }
                    })
                }
            }
        }

    }

    if (state.isShowFilterDialog) {
        TransactionFilterScreen(
            onDismissRequest = {
                viewmodel.onEvent(SearchTransactionUiState.FilterDialog(false))
            },
            onConfirmation = {

            }
        )
    }

}