package com.ubsidifinance.ui.transaction.filter_transaction

import android.os.Build
import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.awaitEachGesture
import androidx.compose.foundation.gestures.awaitFirstDown
import androidx.compose.foundation.gestures.waitForUpOrCancellation
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.graphics.Color.Companion.Transparent
import androidx.compose.ui.input.pointer.PointerEventPass
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import com.ubsidifinance.R
import com.ubsidifinance.base.BaseBottomSheetDialog
import com.ubsidifinance.component.ButtonComponent
import com.ubsidifinance.component.HorizontalDividerComponent
import com.ubsidifinance.component.RequiredTextLabelComponent
import com.ubsidifinance.component.TextComponent
import com.ubsidifinance.component.TextFieldComponent
import com.ubsidifinance.dailog.CustomDatePicker
import com.ubsidifinance.model.RadioButtonModel
import com.ubsidifinance.model.state.TransactionFilterUiState
import com.ubsidifinance.ui.category.CategoryDialog
import com.ubsidifinance.ui.theme.blue316AFF
import com.ubsidifinance.ui.theme.grey_757575
import com.ubsidifinance.ui.theme.grey_9499A5
import com.ubsidifinance.ui.theme.grey_D9D9D9
import com.ubsidifinance.ui.theme.grey_F5F5F5
import com.ubsidifinance.ui.theme.grey_F9F9F9
import com.ubsidifinance.ui.theme.transparent
import com.ubsidifinance.ui.theme.white
import com.ubsidifinance.utils.toFormat

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun TransactionFilterScreen(
    modifier: Modifier = Modifier,
    onDismissRequest: () -> Unit,
    viewmodel: TransactionFilterViewmodel = hiltViewModel(),
    onConfirmation: () -> Unit,
) {

    val state = viewmodel.uiEvent.value
    val scrollState = rememberScrollState()

    val prefixText: (@Composable () -> Unit)? =
        {
            TextComponent(
                text = "£",
                fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                fontSize = 14.sp,
                color = MaterialTheme.colorScheme.secondary
            )
        }

    val suffixDate: (@Composable () -> Unit) = {
        Icon(
            painter = painterResource(R.drawable.ic_calendar),
            contentDescription = "Date Suffix",
            modifier = modifier.size(14.dp),
            tint = MaterialTheme.colorScheme.secondary
        )
    }

    if (state.isShowDatePicker) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            CustomDatePicker(
                title = if (state.isFromDate) "Date From" else "Date To",
                onDismissRequest = {
                    viewmodel.onEvent(TransactionFilterUiState.OnDatePicker(false))
                }, onConfirmation = {
                    viewmodel.onEvent(
                        TransactionFilterUiState.OnDateChange(
                            it.toFormat(),
                            isFromDate = state.isFromDate
                        )
                    )
                })
        }
    }

    if (state.isShowCategoryDialog) {
        CategoryDialog(
            onDismissRequest = {
                viewmodel.onEvent(TransactionFilterUiState.OnCategoryDialog(false))
            },
            onConfirmation = {
                viewmodel.onEvent(TransactionFilterUiState.OnCategoryChange(it))
            }
        )
    }

    BaseBottomSheetDialog(onDismiss = onDismissRequest) {

        Column(
            modifier = modifier
                .verticalScroll(scrollState)
                .padding(24.dp)
                .fillMaxWidth()
        ) {

            HorizontalDividerComponent(
                thickness = 6.dp, modifier = modifier
                    .width(50.dp)
                    .clip(CircleShape)
                    .align(Alignment.CenterHorizontally)
            )

            Row(
                modifier = modifier.fillMaxWidth(),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {

                TextComponent(
                    text = "Transaction Filter",
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    modifier = modifier.padding(top = 15.dp),
                    fontSize = 18.sp,
                    color = MaterialTheme.colorScheme.surfaceDim
                )

                TextComponent(
                    text = "Reset",
                    fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600)),
                    modifier = modifier.padding(top = 15.dp),
                    fontSize = 14.sp,
                    color = MaterialTheme.colorScheme.primary
                )
            }

            Row(
                modifier = modifier
                    .fillMaxWidth()
                    .padding(top = 15.dp)
                    .background(grey_F5F5F5, RoundedCornerShape(8.dp))
                    .padding(3.dp),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                repeat(state.listOfTransactionType.size) {
                    TransactionTypeButton(
                        modifier = modifier.weight(1f),
                        title = state.listOfTransactionType[it],
                        isSelect = state.selectedTransactionIndex == it,
                        onClick = {
                            viewmodel.onEvent(TransactionFilterUiState.OnSelectedIndexChange(it))
                        }
                    )
                }
            }

            TextComponent(
                text = "Date",
                fontFamily = FontFamily(Font(R.font.inter_bold, FontWeight.W600)),
                modifier = modifier.padding(top = 15.dp),
                fontSize = 16.sp,
                color = grey_9499A5
            )

            Spacer(modifier = modifier.height(8.dp))

            repeat(state.listOfDateRange.size) { index ->
                DateRangeItemRow(model = state.listOfDateRange[index], onClick = {
                    viewmodel.onEvent(TransactionFilterUiState.OnSelectedDateRangChange(state.listOfDateRange[index]))
                })
            }

            if (state.isDateRange) {
                Row(modifier = modifier.padding(top = 3.dp)) {
                    Column(
                        modifier = modifier
                            .weight(1f)
                            .padding(end = 12.dp)
                    ) {

                        RequiredTextLabelComponent(
                            text = "From",
                            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                            fontSize = 12.sp,
                            color = MaterialTheme.colorScheme.scrim
                        )

                        TextFieldComponent(
                            value = state.dateFrom,
                            readOnly = true,
                            suffix = suffixDate,
                            newModifier = modifier.pointerInput(state.dateFrom) {
                                awaitEachGesture {
                                    awaitFirstDown(pass = PointerEventPass.Initial)
                                    val upEvent =
                                        waitForUpOrCancellation(pass = PointerEventPass.Initial)
                                    if (upEvent != null) {
                                        viewmodel.onEvent(
                                            TransactionFilterUiState.OnDatePicker(
                                                true,
                                                true
                                            )
                                        )
                                    }
                                }
                            },
                            onValueChange = {

                            })
                    }

                    Column(
                        modifier = modifier
                            .weight(1f)
                    ) {

                        RequiredTextLabelComponent(
                            text = "To",
                            fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                            fontSize = 12.sp,
                            color = MaterialTheme.colorScheme.scrim
                        )

                        TextFieldComponent(
                            value = state.dateTo,
                            suffix = suffixDate,
                            readOnly = true,
                            newModifier = modifier.pointerInput(state.dateTo) {
                                awaitEachGesture {
                                    awaitFirstDown(pass = PointerEventPass.Initial)
                                    val upEvent =
                                        waitForUpOrCancellation(pass = PointerEventPass.Initial)
                                    if (upEvent != null) {
                                        viewmodel.onEvent(
                                            TransactionFilterUiState.OnDatePicker(
                                                true,
                                                false
                                            )
                                        )
                                    }
                                }
                            },
                            onValueChange = {

                            })
                    }

                }
            }

            TextComponent(
                text = "Transaction Amount",
                fontFamily = FontFamily(Font(R.font.inter_bold, FontWeight.W600)),
                modifier = modifier.padding(top = 15.dp),
                fontSize = 16.sp,
                color = grey_9499A5
            )

            Row(modifier = modifier.padding(top = 8.dp)) {
                Column(
                    modifier = modifier
                        .weight(1f)
                        .padding(end = 12.dp)
                ) {

                    TextComponent(
                        text = "Min.",
                        fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                        fontSize = 12.sp,
                        color = MaterialTheme.colorScheme.scrim
                    )

                    TextFieldComponent(
                        value = state.minTransaction,
                        newModifier = modifier,
                        keyboardOptions = KeyboardOptions(
                            imeAction = ImeAction.Done,
                            keyboardType = KeyboardType.Decimal
                        ),
                        prefix = prefixText,
                        onValueChange = {
                            if (it.length <= 5)
                                viewmodel.onEvent(TransactionFilterUiState.OnMinTransactionChange(it))
                        })
                }

                Column(
                    modifier = modifier
                        .weight(1f)
                ) {

                    TextComponent(
                        text = "Max.",
                        fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                        fontSize = 12.sp,
                        color = MaterialTheme.colorScheme.scrim
                    )

                    TextFieldComponent(
                        value = state.maxTransaction,
                        newModifier = modifier,
                        keyboardOptions = KeyboardOptions(
                            imeAction = ImeAction.Done,
                            keyboardType = KeyboardType.Decimal
                        ),
                        prefix = prefixText,
                        onValueChange = {
                            if (it.length <= 5)
                                viewmodel.onEvent(TransactionFilterUiState.OnMaxTransactionChange(it))
                        })
                }

            }

            TextComponent(
                text = "Category",
                fontFamily = FontFamily(Font(R.font.inter_bold, FontWeight.W600)),
                modifier = modifier.padding(top = 15.dp, bottom = 8.dp),
                fontSize = 16.sp,
                color = grey_9499A5
            )

            Row(
                modifier = modifier
                    .fillMaxWidth()
                    .border(1.dp, grey_D9D9D9, RoundedCornerShape(10.dp))
                    .clip(RoundedCornerShape(10.dp))
                    .clickable(onClick = {
                        viewmodel.onEvent(TransactionFilterUiState.OnCategoryDialog(true))
                    })
                    .padding(12.dp),
                verticalAlignment = Alignment.CenterVertically
            ) {
                Image(
                    painter = painterResource(state.selectedCategory.image),
                    contentDescription = "",
                    modifier = modifier
                        .background(grey_F9F9F9, CircleShape)
                        .size(30.dp)
                        .padding(8.dp)
                )

                TextComponent(
                    text = state.selectedCategory.title,
                    fontFamily = FontFamily(Font(R.font.inter_medium, FontWeight.W500)),
                    modifier = modifier
                        .padding(start = 12.dp, end = 8.dp)
                        .weight(1f),
                    fontSize = 12.sp,
                    color = MaterialTheme.colorScheme.inversePrimary
                )

                Icon(
                    painter = painterResource(R.drawable.ic_down_arrow),
                    contentDescription = "",
                    modifier = modifier.rotate(270f),
                    tint = grey_9499A5
                )

            }

            ButtonComponent(
                text = "Apply",
                modifier = modifier.padding(top = 15.dp),
                onClick = {
                    onConfirmation.invoke()
                }
            )
        }
    }
}

@Composable
fun TransactionTypeButton(
    modifier: Modifier = Modifier,
    title: String,
    isSelect: Boolean = false,
    onClick: () -> Unit
) {
    TextComponent(
        text = title,
        modifier = modifier
            .background(if (isSelect) blue316AFF else transparent, RoundedCornerShape(8.dp))
            .clip(RoundedCornerShape(8.dp))
            .clickable(onClick = onClick)
            .padding(vertical = 6.dp),
        style = TextStyle(
            color = if (isSelect) white else grey_757575,
            fontSize = 16.sp,
            fontFamily = FontFamily(
                Font(
                    R.font.inter_regular,
                    if (isSelect) FontWeight.W600 else FontWeight.W400
                )
            ),
            textAlign = TextAlign.Center
        ),
    )
}

@Composable
fun DateRangeItemRow(
    modifier: Modifier = Modifier,
    model: RadioButtonModel,
    onClick: () -> Unit
) {
    val interactionSource = remember { MutableInteractionSource() }

    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .clickable(onClick = onClick, indication = null, interactionSource = interactionSource)
            .padding(bottom = 12.dp)
    ) {

        TextComponent(
            modifier = modifier.weight(1f),
            text = model.title,
            fontSize = 14.sp,
            color = MaterialTheme.colorScheme.surfaceDim,
            fontFamily = FontFamily(Font(R.font.inter_semi_bold, FontWeight.W600))
        )

        Box(
            modifier = modifier
                .size(20.dp)
                .border(1.dp, if (model.isSelected) blue316AFF else grey_D9D9D9, CircleShape)
                .padding(3.dp)
                .background(if (model.isSelected) blue316AFF else Transparent, CircleShape)
        )
    }
}


