package com.ubsidifinance.utils

import android.content.Context
import android.net.Uri
import android.os.Build
import android.util.Log
import android.util.Patterns
import android.widget.Toast
import androidx.camera.core.ImageCapture
import androidx.camera.core.ImageCaptureException
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.content.ContextCompat
import androidx.lifecycle.Lifecycle
import androidx.navigation.NavController
import com.google.i18n.phonenumbers.NumberParseException
import com.google.i18n.phonenumbers.PhoneNumberUtil
import com.ubsidifinance.model.ErrorResponse
import kotlinx.coroutines.suspendCancellableCoroutine
import kotlinx.serialization.json.Json
import okhttp3.ResponseBody
import java.io.File
import java.text.SimpleDateFormat
import java.time.LocalDate
import java.time.format.DateTimeFormatter
import java.util.Date
import java.util.Locale

fun NavController.isResume() =
    this.currentBackStackEntry?.lifecycle?.currentState == Lifecycle.State.RESUMED

fun String?.toNonNullString(): String {
    if (this == null || this.equals("null", true))
        return ""
    return this
}

fun LocalDate?.toFormat(format: String = dmmmy): String {
    return try {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val formatter = DateTimeFormatter.ofPattern(format)
            this?.format(formatter) ?: ""
        } else {
            ""
        }
    } catch (e: IllegalArgumentException) {
        e.printStackTrace()
        ""
    } catch (e: Exception) {
        e.printStackTrace()
        ""
    }
}

fun String.toLocalDate(
    format: String = dmmmy,
    locale: Locale = Locale.ENGLISH
): LocalDate? {
    return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
        try {
            val formatter = DateTimeFormatter.ofPattern(format, locale)
            LocalDate.parse(this, formatter)
        } catch (e: Exception) {
            e.printStackTrace()
            null
        }
    } else {
        null
    }
}

fun String.isValidateEmpty(): Boolean {
    return this.trim().toNonNullString() == ""
}

fun String.isValidEmail(): Boolean {
    return Patterns.EMAIL_ADDRESS.matcher(this).matches()
}

fun Long.toFormat(format: String = dmy): String {
    val formatter = SimpleDateFormat(format, Locale.getDefault())
    return formatter.format(Date(this))
}

suspend fun Context.getCameraProvider(): ProcessCameraProvider =
    suspendCancellableCoroutine { cont ->
        val future = ProcessCameraProvider.getInstance(this)
        future.addListener({
            cont.resume(future.get(), null)
        }, ContextCompat.getMainExecutor(this))
    }

fun ImageCapture.takePictureAndCheckFace(
    context: Context,
    photoFile: File,
    onNoFaceDetected: (Uri) -> Unit,
    onFaceDetected: () -> Unit = {},
    onError: (ImageCaptureException) -> Unit = {}
) {
    val outputOptions = ImageCapture.OutputFileOptions.Builder(photoFile).build()

    this.takePicture(
        outputOptions,
        ContextCompat.getMainExecutor(context),
        object : ImageCapture.OnImageSavedCallback {
            override fun onImageSaved(outputFileResults: ImageCapture.OutputFileResults) {
                Log.d("CameraX", "Image saved to ${photoFile.absolutePath}")

                detectFaceInImage(
                    imageFile = photoFile,
                    context = context,
                    onResult = { faceFound ->
                        if (!faceFound) {
                            onNoFaceDetected(Uri.fromFile(photoFile))
                        } else {
                            onFaceDetected()
                        }
                    })
            }

            override fun onError(exception: ImageCaptureException) {
                onError(exception)
            }
        }
    )
}

internal fun List<Float>.toPercent(): List<Float> {
    return this.map { item ->
        item * 100 / this.sum()
    }
}

fun String.toInitials(): String {
    return this
        .split(" ")
        .filter { it.isNotBlank() }
        .map { it.first().uppercaseChar() }
        .joinToString("")
}

fun Context.showToast(message: String?) {
    val toast = Toast.makeText(this, message.toNonNullString(), Toast.LENGTH_SHORT)
    toast.show()
}

fun ResponseBody.parseError(): ErrorResponse {
    return try {
        val json = Json { ignoreUnknownKeys = true }
        json.decodeFromString(ErrorResponse.serializer(), this.string())
    } catch (e: Exception) {
        e.printStackTrace()
        ErrorResponse(message = "Something went wrong")
    }
}

fun isMobileNumberValid(mobileNumber: String, countryCode: String): Boolean {
    return try {
        PhoneNumberUtil.getInstance()
        val phoneNumber =
            PhoneNumberUtil.getInstance().parse(
                mobileNumber.trim(),
                countryCode.uppercase().trim()
            )
        PhoneNumberUtil.getInstance().isValidNumber(phoneNumber)
    } catch (exception: Exception) {
        exception.printStackTrace()
        if (exception !is NumberParseException) {
            Log.e("TAG", "isMobileNumberValid: ${exception.message}")
        }
        false
    }
}

fun getExampleMobileNumber(countryCode: String, formatNumber: Boolean = false): String {
    return try {
        val phoneUtil = PhoneNumberUtil.getInstance()
        val exampleMobileNumber =
            phoneUtil.getExampleNumberForType(
                countryCode.uppercase().trim(),
                PhoneNumberUtil.PhoneNumberType.MOBILE
            )
        exampleMobileNumber?.let {
            if (formatNumber) {
                val countryDialCode = "tel:+${exampleMobileNumber.countryCode}-"
                val fullFormatted = phoneUtil.format(it, PhoneNumberUtil.PhoneNumberFormat.RFC3966)
                    .removePrefix(countryDialCode)
                val regex = Regex("[1-9]")
                val raw = fullFormatted.replace(regex, "0")
                raw
            } else {
                it.nationalNumber.toString()
            }
        } ?: ""
    } catch (exception: Exception) {
        exception.printStackTrace()
        ""
    }
}

fun formatPhoneNumberWithoutCountryCode(
    input: String,
    countryCode: String,
    format: PhoneNumberUtil.PhoneNumberFormat = PhoneNumberUtil.PhoneNumberFormat.INTERNATIONAL
): String {
    return try {
        val phoneUtil = PhoneNumberUtil.getInstance()
        val parsedNumber = phoneUtil.parse(input, countryCode.uppercase().trim())

        if (phoneUtil.isValidNumber(parsedNumber)) {
            val fullFormatted = phoneUtil.format(parsedNumber, format)

            val countryDialCode = "+${parsedNumber.countryCode}"
            fullFormatted.removePrefix(countryDialCode).trim()
        } else {
            input
        }
    } catch (e: Exception) {
        input
    }
}

